//= require <dSS/GroupDialogs>

Ext.namespace('dSS');

dSS.GroupView = Ext.extend(Ext.DataView, {
    initComponent: function() {

        var groupTemplate = new Ext.XTemplate(
            '<tpl for=".">',
            '<div class="zone-wrap" id="group-{id}">',
            '<tpl if="css.length &gt; 0">',
                '<table><tbody><tr>',
                    '<td>',
                        '<div class="{css} x-icon-combo-icon"></div></td>',
                    '<td>{name}</td>',
                '</tr></tbody></table>',
            '</tpl>',
            '<tpl if="(css.length == 0)">',
                '<span>{name}</span>',
            '</tpl>',
            '</div>',
            '</tpl>',
            '<div class="x-clear"></div>');

        Ext.apply(this, {
            bubbleEvents: [ 'groupselection' ],
            store: dSS.data.Loader.getGroupStore(),
            tpl: groupTemplate,
            singleSelect: true,
            multiSelect: false,
            layout: 'fit',
            style: "overflow: auto",
            overClass: 'x-view-over',
            itemSelector: 'div.zone-wrap',
            emptyText: dSS._("No groups to display")
        });

        dSS.GroupView.superclass.initComponent.apply(this, arguments);

        dSS.data.Loader.getDeviceStore().on('load', function() {
                this.updateNumDevicesHeader();
            },
            this);

        this.on('selectionchange', function(dv, nodes) {
            if (nodes.length > 0) {
                var selectedGroups = this.getSelectedRecords();
                if (selectedGroups.length > 0) {
                    this.lastSelectedGroupID = selectedGroups[0].get('id');
                 }
            } else {
                if (this.lastSelectedGroupID !== undefined) {
                    var rec = this.getStore().getById(this.lastSelectedGroupID);
                    if (rec !== undefined) {
                        this.select(rec);
                    }
                }
            }

            var selection = this.getSelectedRecords();
            if (selection.length > 0) {
                var record = selection[0];
                var state = (record.get('id') < dSS.config.UserGroupStartIndex);
                this.refOwner.getTopToolbar().tbRemoveGroup.setDisabled(state);
                this.refOwner.getTopToolbar().tbEditGroup.setDisabled(state);
                this.fireEvent('groupselection', record, state);
            }
            this.filterDevices();
            this.updateNumDevicesHeader();
        },
        this);

        // do not allow to select the separator
        this.on('beforeselect', function(dv, node, selection) {
                if (node.id == "group--1") {
                    return false;
                }

                return true;
            },
            this);

        dSS.MessageBus.on('structureloaded', function() {
            if (!this.ownerCt.isVisible()) {
                return;
            }

            if (this.getStore().getCount() === 0) {
                this.getStore().clearFilter();
                this.updateNumDevicesHeader();
                return;
            }

            if (this.lastSelectedZoneID !== undefined) {
                var rec = this.getStore().getById(this.lastSelectedZoneID);
                if (rec !== undefined) {
                    this.select(rec);
                }
            } else {
                this.select(this.store.getById(0));
            }
        },
        this);

        this.on('render', function() {
            var groupView = this;
            this.dropZone = new Ext.dd.DropZone(groupView.getEl(), {
                ddGroup: "groupDeviceDD",
                containerScroll: true,
                getTargetFromEvent: function(e) {
                    return e.getTarget(groupView.itemSelector);
                },
                onNodeEnter: function(target, dd, e, data) {
                    Ext.fly(target).addClass('my-row-highlight-class');
                },
                onNodeOut: function(target, dd, e, data) {
                    Ext.fly(target).removeClass('my-row-highlight-class');
                },
                onNodeOver: function(target, source, e, data) {
                    var record = groupView.getRecord(target);
                    var grpSelection = groupView.getSelectedRecords();
                    var sourceGroup = 0;
                    if (grpSelection.length > 0) {
                        sourceGroup = grpSelection[0].id;
                    }

                    // prevent self-drops
                    if ((sourceGroup < dSS.config.UserGroupStartIndex) &&
                        (record.get('id') < dSS.config.UserGroupStartIndex)) {
                        return Ext.dd.DropZone.prototype.dropNotAllowed;
                    }

                    // if source check passed above, prevent drops to all non
                    // user groups except special case "0" which surprisingly
                    // equals to "remove from group"
                    if ((record.get('id') > 0) &&
                        (record.get('id') < dSS.config.UserGroupStartIndex)) {
                        return Ext.dd.DropZone.prototype.dropNotAllowed;
                    }

                    // special case: drop to group 0 means "remove"
                    if (record.get('id') === 0) {
                        return Ext.dd.DropZone.prototype.dropAllowed;
                    }

                    var alreadyInGroupCounter = 0;
                    var dropOK = true;
                    Ext.each(data.selections, function(device) {
                        var selectedColor = dSS.util.getDeviceColor(device);
                        var pid = dSS.util.parseProductID(
                                                    device.get('productID'));
                        var devType = pid[0];
                        if ((record.get('color') !== selectedColor) &&
                           (devType !== dSS.config.deviceTypeNum.AKM)) {
                            dropOK = false;
                            return false;
                        }

                        if (device.get('groups').indexOf(record.get('id'))
                                !== -1) {
                            alreadyInGroupCounter++;
                        }
                    });

                    if (dropOK === false) {
                        return Ext.dd.DropZone.prototype.dropNotAllowed;
                    }

                    // all selected devices are already in target group
                    if (alreadyInGroupCounter == data.selections.length) {
                        return Ext.dd.DropZone.prototype.dropNotAllowed;
                    }

                    return Ext.dd.DropZone.prototype.dropAllowed;
                },
                onNodeDrop: function(target, dd, e, data) {
                    var record = groupView.getRecord(target);
                    var grpSelection = groupView.getSelectedRecords();
                    var sourceGroup = 0;
                    if (grpSelection.length > 0) {
                        sourceGroup = grpSelection[0].id;
                    }

                    // prevent self-drops
                    if ((sourceGroup < dSS.config.UserGroupStartIndex) &&
                        (record.get('id') < dSS.config.UserGroupStartIndex)) {
                        return;
                    }

                    // if source check passed above, prevent drops to all non
                    // user groups except special case "0" which surprisingly
                    // equals to "remove from group"
                    if ((record.get('id') > 0) &&
                        (record.get('id') < dSS.config.UserGroupStartIndex)) {
                        return;
                    }

                    var groupColor = record.get('color');
                    var rmRecords = [];

                    Ext.each(data.selections, function(device) {
                        // skip non presnet devices
                        if (device.get('isPresent') === false) {
                            return true;
                        }

                        // selection takes care of only allowing same color
                        // devices, so if one does not match we can expect
                        // the whole selection to be bogus and stop iteration
                        var col = dSS.util.getDeviceColor(device);
                        var pid = dSS.util.parseProductID(
                                                    device.get('productID'));
                        var devType = pid[0];
                        if (((col !== groupColor) &&
                             (devType !== dSS.config.deviceTypeNum.AKM)) &&
                             (record.get('id') !== 0)) {
                            return false;
                        }

                        // group 0 - remove mode
                        if (record.get('id') === 0) {
                            rmRecords.push(device);
                            return true;
                        }

                        // skip devices that are already in the group
                        if (device.get('groups').indexOf(record.get('id'))
                                !== -1) {
                            return true;
                        }

                        Ext.Ajax.request({
                            url: '/json/structure/groupAddDevice',
                            disableCaching: true,
                            method: "GET",
                            scope: groupView,
                            params: {
                                deviceID: device.get('dsid'),
                                groupID: record.get('id')
                            },
                            success: function(result, request) {
                                try {
                                    var json = Ext.util.JSON.decode(result.responseText);
                                    if (json.ok) {
                                        var store = dSS.data.Loader.getDeviceStore();
                                        if (json.result.action === "update") {
                                            var i;
                                            var zoneStore =
                                                dSS.data.Loader.getZoneStore();
                                            for (i = 0; i < json.result.devices.length; i++) {
                                                var zone = zoneStore.getById(json.result.devices[i].zoneID);
                                                if (zone !== undefined) {
                                                    json.result.devices[i].zoneName = zone.get('name');
                                                }
                                                json.result.devices[i].firstSeen =
                                                    Date.parseDate(
                                                       json.result.devices[i].firstSeen,
                                                       "Y-m-d H:i:s", true);
                                                json.result.devices[i].inactiveSince =
                                                    Date.parseDate(
                                                        json.result.devices[i].inactiveSince,
                                                        "Y-m-d H:i:s", true);
                                            }
                                            store.loadData(json.result, true);
                                            dSS.MessageBus.fireEvent(
                                                'statusmessage',
                                                dSS.gt.strargs(
                                                    dSS._("Device %1 was added"+
                                                        " to group %2"),
                                                    [
                                                        '&quot;' +
                                                        dSS.util.getName(
                                                            device.get('name'))+
                                                        '&quot; (' +
                                                        dSS.util.shortDSID(
                                                            device.get('dsid'))+
                                                        ')',
                                                        '&quot;' +
                                                        record.get('name') +
                                                        '&quot;'
                                                    ]
                                                )
                                            );
                                        }
                                    } else {
                                        Ext.Msg.alert(dSS._("Error"),
                                            dSS._("Device") +
                                            ' ' + dSS.util.shortDSID(
                                                    device.get('dsid')) +
                                            ' ' + dSS._("could not be added " +
                                                "to group") + ' ' +
                                            record.get('name') +
                                            ': (' + json.message + ')');
                                    }
                                } catch(err) {
                                    Ext.Msg.alert(
                                        dSS._("Error"), dSS._("Device") + ' ' +
                                        dSS.util.shortDSID(
                                            device.get('dsid')) + ' ' +
                                        dSS._("could not be added to group") +
                                        ' ' + record.get('name') +
                                        ': (' + err + ')');
                                    dSS.MessageBus.fireEvent('reloadapartment');
                                }
                            },
                            failure: function(result, request) {
                                Ext.Msg.alert(
                                    dSS._("Error"), dSS._("Device") +  ' ' +
                                    dSS.util.shortDSID(
                                        device.get('dsid')) + ' ' +
                                    dSS._("could not be added to group") +
                                    ' ' + record.get('name'));
                                dSS.MessageBus.fireEvent('reloadapartment');
                            }
                        });
                    });

                    // this is actually a remove group action, no adding
                    // took place
                    if (rmRecords.length > 0) {
                        var grpSelection = groupView.getSelectedRecords();
                        if (grpSelection.length > 0) {
                            groupView.findParentByType('dssgroupbrowser').devicePanel.removePresentFromGroup(rmRecords, grpSelection[0].id);
                        }
                    }
 
                    return true;
                }
            });
        },
        this);

        this.on('contextmenu', function(view, index, node, event) {
            event.preventDefault();
            event.stopEvent();
            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            this.select(index);
            var sel = this.getSelectedRecords();
            var record = sel[0];

            var menuItem = new Ext.menu.Item({
                text: dSS._("Create new user group"),
                iconCls: 'addUserGroupIcon',
                handler: this.createGroup.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);

            var modify = true;
            if (record.get('id') < dSS.config.UserGroupStartIndex) {
                modify = false;
            }
            menuItem = new Ext.menu.Item({
                text: dSS._("Edit user group"),
                iconCls: 'editUserGroupIcon',
                disabled: !modify,
                handler: this.editGroup.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                text: dSS._("Remove user group"),
                iconCls: 'removeUserGroupIcon',
                disabled: !modify,
                handler: this.removeGroup.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);


            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);
    },

    filterDevices: function() {
        var selectedGroups = this.getSelectedRecords();
        var store = dSS.data.Loader.getDeviceStore();

        if ((selectedGroups.length === 0) ||
            (selectedGroups[0].get('id') == 0)) {
            store.filterBy(function(record) {
                return record.get('isPresent');
            });
            return;
        }

        store.filterBy(function(record) {
            var i;

            if (record.get('isPresent') === false) {
                return false;
            }

            for (i = 0; i < selectedGroups.length; i++) {
                if (record.get('groups').indexOf(selectedGroups[i].get('id'))
                    !== -1) {
                    return true;
                }
            }
            return false;
        });
    },

    updateNumDevicesHeader: function() {
        var count = dSS.data.Loader.getDeviceStore().getRange().length;
        var selectedGroups = this.getSelectedRecords();
        var title = '';
        if ((selectedGroups.length === 0) ||
            (selectedGroups[0].get('id') === 0)) {
            title = dSS._("Devices in all groups");
        } else {
            title = dSS._("Devices in group") + " \"" + selectedGroups[0].get('name') + " - #" + selectedGroups[0].get('id') + "\"";
        }
        this.findParentByType('dssgroupbrowser').devicePanel.setTitle("<span class=\"x-panel-header-text\">" + title + "</span><span style=\"float: right;\" class=\"x-panel-header-text\">" + count + " " + dSS._("Device", "Devices", count) + "</span>");
    },

    removeGroup: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No group selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        var deviceStore = dSS.data.Loader.getDeviceStore();

        if (deviceStore.getRange().length > 0) {
            Ext.Msg.alert(dSS._("Error"),
                          dSS._("Can not delete a non empty group, please " +
                                "remove all devices from the group first."));
            return;
        }

        Ext.Ajax.request({
            url: '/json/structure/removeGroup',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                zoneID: 0, // global group
                groupID: record.get('id')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        this.getStore().remove(record);
                        this.select(0);
                        dSS.MessageBus.fireEvent('statusmessage',
                          dSS._("Group") + ' &quot;' +
                             record.get('name') +
                            '&quot; ' + dSS._("was removed"));
                    } else {
                        Ext.Msg.alert(dSS._("Error"),
                                     dSS._("Could not remove group") + ': ' +
                                     jsonData.message);
                    }
                } catch(error) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not remove group") + ': ' +
                                  error);
                }
            },
            failure: function(result, request) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not remove group"));
            }
        });
    },
    editGroup: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No group selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            title: dSS._("Edit user group"),
            rowSel: this,
            jumpMode: false,
            dialogType: dSS.config.dialogTypes.EDIT_GROUP_DIALOG,
            maskElement: this.findParentByType(Ext.Viewport).el,
            width: 300
        });

        win.show();
    },
    createGroup: function() {
        var ugCount = 0;
        this.getStore().each(function(record) {
            if (record.get('id') >= dSS.config.UserGroupStartIndex) {
              ugCount++;
            }
            if (ugCount >= dSS.config.UserGroupMaxCount) {
                return false;
            }
            return true;
        });

        if (ugCount >= dSS.config.UserGroupMaxCount) {
          Ext.Msg.alert(dSS._("Warning"),
                        dSS._("Maximum number of user groups reached"));
          return;
        }

        var win = new dSS.devprops.DevicePropertiesWindow({
            rowSel: this,
            title: dSS._("Create new user group"),
            dialogType: dSS.config.dialogTypes.CREATE_GROUP_DIALOG,
            maskElement: this.findParentByType(Ext.Viewport).el,
            width: 300
        });

        win.show();
    }
});

Ext.reg('dssgroupview', dSS.GroupView);

