Ext.namespace("dSS", "dSS.config", "dSS.devprops", "dSS.gthelper");

dSS.devprops.CreatePanelForDeviceProperties = function(record, featureConfig,
                                                      scope) {
    var devInfo = dSS.util.parseProductID(record.get('productID'));
    var type = devInfo[0];
    var number = devInfo[1];
    var cls = dSS.util.parseFunctionID(record.get('functionID'));
    var stdForms = [];
    var advancedForms = [];
    var loaders = [];
    var showDeviceInfo = true;
    var matrix;
    var extraTitle;

    try {
        matrix = dSS.devprops.GetVisibleProperties(cls[0], type, number);
    } catch (error) {
        // for now we can safely assume all as invisible
        matrix = error;
    }

    if (featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
        var addloader = false;
        var addLocalPrioLoader = false;
        var color = cls[0];

        if (matrix.akmsensor === true) {
            stdForms.push(
                {
                    xtype: 'dssakmsensorconfigform',
                    device: record
                });
            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.AKM_BINARY_INPUTS,
                         url: '/json/device/getBinaryInputs',
                         params:
                         {
                             dsid: record.get('dsid')
                         }
                     }));
        }
        if (matrix.akminput === true) {
            stdForms.push(
                {
                    xtype: 'dssakminputconfigform',
                    device: record
                });
            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.AKM_INPUT_PROPERTY,
                         url: '/json/property/getString',
                         params:
                         {
                             path: '/apartment/zones/zone0/devices/' +
                                   record.get('dsid') +
                                   '/AKMInputProperty'
                         }
                     }));

        }

        if (matrix.akmdelay === true) {
            stdForms.push(
                {
                    xtype: 'dssakmdelayconfigform',
                    dsid: record.get('dsid')
                });
            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.AKM_DELAY_CONFIG,
                         url: '/json/device/getAKMInputTimeouts',
                         params:
                         {
                             dsid: record.get('dsid')
                         }
                     }));
        }

        if ((matrix.jokerconfig === true) &&
            (cls[0] === dSS.config.groups.JOKER)) {
            color = dSS.util.getJokerGroup(record.get('groups'));
            // assume it's not yet configured
            if (color === null) {
                color = dSS.config.groups.JOKER;
            }
            stdForms.push(
                {
                    xtype: 'dssdevjokerconfigform',
                    dsid: record.get('dsid'),
                    device: record,
                    product: devInfo[0],
                    group: color
                });
        }

        if (matrix.optypeconfig === true) {
            stdForms.push(
                {
                    xtype: 'dssdevoptypeform',
                    dsid: record.get('dsid')
                });
            addloader = true;
        }

        if ((matrix.pushbutton === true) &&
            (record.get('productRevision') >= dSS.config.PushbuttonMinFW)) {
            stdForms.push(
                {
                    xtype: 'dssdevgrouppushbuttonform',
                    dsid : record.get('dsid'),
                    product: type,
                    matrix: matrix,
                    group: color
                });

            if (matrix.pushbadvanced === true) {
                advancedForms.push(
                    {
                        xtype: 'dssdevgrouppushbuttonadvancedform',
                        dsid : record.get('dsid'),
                        matrix: matrix,
                        group: color
                     });
            } else {
                stdForms.push(
                {
                    xtype: 'dssdevgrouppushbuttonadvancedform',
                    dsid : record.get('dsid'),
                    matrix: matrix,
                    group: color,
                    disabled: true,
                    hidden: true
                });
            }

            addloader = true;
            addLocalPrioLoader = true;
        }

        if ((matrix.pushbutton === true) &&
            (matrix.twowayconfig === true) &&
            (dSS.util.deviceHasMultipleButtons(record.get('functionID')) === true) &&
            ((record.get('buttonInputIndex') === 0) || (record.get('buttonInputIndex') === 2))) {
            stdForms.push(
            {
                xtype: 'dssdevmultibuttonconfigform',
                dsid : record.get('dsid'),
                buttonIndex : record.get('buttonInputIndex'),
                buttonConfig: record.get('buttonInputMode'),
                buttonID: record.get('buttonID'),
                group: color,
                matrix: matrix
            });
        }
        if (matrix.outmode === true) {
            stdForms.push(
                {
                    xtype: 'dssdevoutputmodeform',
                    dsid : record.get('dsid')
                });
            addloader = true;
        }

        if (addloader === true) {
            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.COMBINED_PB_OUTMODE,
                         url: '/json/property/query',
                         params:
                         {
                            query: '/apartment/zones/zone0/devices/' +
                                 record.get('dsid') +
                                 '(outputMode)/button(id,setsLocalPriority,callsPresent,activeGroup)'
                         }
                     }));
        }

        if (addLocalPrioLoader === true) {
            loaders.push(new dSS.devprops.Loader({
                         loaderId: dSS.devprops.Loaders.SCENE_CONFIGURATION,
                         url: '/json/device/getSceneMode',
                         params:
                         {
                             sceneID : dSS.config.scenes.LOCAL_ON,
                             dsid: record.get('dsid')
                         }
                     }));
        }
    }

    var useTabs = false;
    var jumpMode = false;
    var additionalInfo = [];
    var tabHeight;
    var tabWidth;
    if (featureConfig === dSS.config.FeatureConfig.SCENE) {
        tabHeight = 160;
        var scene;
        useTabs = true;
        tabWidth = dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth') + 13;
        jumpMode = true;
        additionalInfo.push(
        {
            key: dSS._("Output mode"),
            value: dSS._(dSS.config.OutModeStrings[record.get('outputMode')])
        });
        try {
            scene = scope.findParentByType('dssscenespanel').getSelectedScene();
        } catch (err) {
            Ext.Msg.alert(dSS._("Error"), err);
            return;
        }

        var sceneStore = dSS.data.Loader.getSceneStore();
        var sceneId = record.get('zoneId') + '-' + cls[0] + '-' + scene;
        var sceneRec = sceneStore.getById(sceneId);
        var sceneName = "";
        if (sceneRec !== undefined) {
            sceneName = sceneRec.get('name');
        }

        if (sceneName.length > 0) {
            extraTitle = ' - ' + sceneName;
        } else {
            extraTitle = ' - ' + dSS._(dSS.config.sceneNames[scene]);
        }

        if (dSS.devprops.CheckShowSceneProperties(matrix) === true) {
            stdForms.push({
                        xtype: 'dssdevscenepropsform',
                        deviceRecord: record,
                        scene: scene,
                        visibilityMatrix: matrix
                   });
            advancedForms.push({
                        xtype: 'dssdevscenepropsadvancedform',
                        deviceRecord: record,
                        scene: scene,
                        visibilityMatrix: matrix
                   });
            if (record.get('isPresent') === true) {
                loaders.push(new dSS.devprops.Loader({
                             loaderId: dSS.devprops.Loaders.SCENE_MODE,
                             url: '/json/device/getSceneMode',
                             params:
                             {
                                sceneID : scene,
                                dsid: record.get('dsid')
                             }
                         }));
            }
        }

        if ((scene >= dSS.config.scenes.PRESENT) ||
            (scene == dSS.config.scenes.PANIC)) {
            tabHeight = 340;
            stdForms.push({
                    xtype: 'dssdevicesettingswrapper',
                    deviceRecord: record,
                    scene: scene,
                    visibilityMatrix: matrix
            });

            if ((record.get('isPresent') === true) &&
                (matrix.outvalue8 || matrix.shadeposition)) {
                loaders.push(new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.SCENE_VALUE,
                    url: '/json/device/getSceneValue',
                    params:
                    {
                        sceneID: scene,
                        dsid: record.get('dsid')
                    }
                }));
            }
        }
    }

    var reason = dSS._("Modifying settings for this device is not allowed!");
    var confLock = record.get('configurationLocked');
    var oemState = record.get('OemStatus');
    if (confLock && ((oemState === 'Unknown') || (oemState === 'Loading'))) {
        reason = dSS._("Modifying settings for this device is not allowed, because device configuration has not been read out yet!");
    }

    var retObj =
    {
        panel:    new dSS.devprops.DevicePropertiesPanel({
                        dsid: record.get('dsid'),
                        name: record.get('name'),
                        showDefaults: true,
                        additionalInfo: additionalInfo,
                        standardForms: stdForms,
                        advancedForms: advancedForms,
                        loaders: loaders,
                        useTabs: useTabs,
                        tabHeight: tabHeight,
                        tabWidth: tabWidth,
                        stayMasked: confLock,
                        maskReason: reason
                    }),
        extraTitle: extraTitle
    };
    return retObj;
};


dSS.devprops.CreatePanelForShadeProperties = function(record) {
    var devInfo = dSS.util.parseProductID(record.get('productID'));
    var type = devInfo[0];
    var number = devInfo[1];
    var cls = dSS.util.parseFunctionID(record.get('functionID'));
    var matrix;
    try {
        matrix = dSS.devprops.GetVisibleProperties(cls[0], type, number);
    } catch (error) {
        // for now we can safely assume all as invisible
        matrix = error;
    }

    if (matrix.shadeprops !== true) {
        Ext.Msg.alert(dSS._("Error"), dSS._("Shade properties can only be configured for grey klemme L devices"));
        return;
    }

    var shadeloaders =
        [
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.SHADE_OPEN_CLOSE,
                url: '/json/device/getConfigWord',
                params:
                {
                    dsid: record.get('dsid'),
                    'class': 3,
                    index: 0x32
                }
            }),
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.SHADE_MTIME_BLINK,
                url: '/json/device/getConfig',
                params:
                {
                    dsid: record.get('dsid'),
                    'class': 3,
                    index: 0x11
                }
            }),
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.SHADE_CLOSE_OPEN,
                url: '/json/device/getConfigWord',
                params:
                {
                    dsid: record.get('dsid'),
                    'class': 3,
                    index: 0x30
                }
            }),
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.SHADE_FTEND_TRESHOLDENDSW,
                url: '/json/device/getConfigWord',
                params:
                {
                    dsid: record.get('dsid'),
                    'class': 3,
                    index: 0x38
                }
            })
        ];

    if (matrix.motiontimefins === true) {
        shadeloaders.push(
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.SHADE_MTIME_FINS,
                url: '/json/device/getConfigWord',
                params:
                {
                    dsid: record.get('dsid'),
                    'class': 3,
                    index: 0x3e
                }
            })
        );
    }

    shadeloaders = shadeloaders.concat(
         [
             new dSS.devprops.Loader({
                 loaderId: dSS.devprops.Loaders.SHADE_NUMST_MTIME_FTUNE,
                 url: '/json/device/getConfigWord',
                 params:
                 {
                     dsid: record.get('dsid'),
                     'class': 3,
                     index: 0x3a
                 }
             }),
             new dSS.devprops.Loader({
                 loaderId: dSS.devprops.Loaders.SHADE_CALIBRATION_ALLOWED,
                 url: '/json/device/getConfig',
                 params:
                 {
                     dsid: record.get('dsid'),
                     'class': 3,
                     index: 0x3d
                 }
             }),
             new dSS.devprops.Loader({
                 loaderId: dSS.devprops.Loaders.SHADE_DIR_TRAVEL_RTIME_CH,
                 url: '/json/device/getConfigWord',
                 params:
                 {
                     dsid: record.get('dsid'),
                     'class': 3,
                     index: 0x36
                 }
             })
         ]
    );

    var standardForms =
    [
        {
            xtype: 'shadepropsform',
            dsid: record.get('dsid'),
            matrix: matrix
        }
    ];

    var advancedForms =
    [
        {
            xtype: 'shadepropsadvancedform',
            dsid: record.get('dsid')
        }
    ];

    var retObj =
    {
        panel : new dSS.devprops.DevicePropertiesPanel({
                    dsid: record.get('dsid'),
                    name: record.get('name'),
                    showDefaults: true,
                    standardForms: standardForms,
                    advancedForms: advancedForms,
                    loaders: shadeloaders,
                    useTabs: false
                }),
        extraTitle: dSS._("shade")
    };
    return retObj;
};

dSS.devprops.CreatePanelForConsumptionSettings = function(record) {
        var devInfo = dSS.util.parseProductID(record.get('productID'));
        var type = devInfo[0];
        var number = devInfo[1];
        var cls = dSS.util.parseFunctionID(record.get('functionID'));
        var matrix;
        try {
            matrix = dSS.devprops.GetVisibleProperties(cls[0], type, number);
        } catch (error) {
            // for now we can safely assume all as invisible
            matrix = error;
        }

        if (matrix.consumption !== true) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Consumption thresholds can only be configured for klemme L and ZWS devices"));
            return;
        }

        var icon = 'titleIcon' + dSS.util.genCSSName(record);

        var thresholdloaders =
            [
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.CONSUMPTION_TRESHOLD_CURRENT,
                    url: '/json/device/getSensorValue',
                    params:
                    {
                        dsid: record.get('dsid'),
                        sensorIndex: 2
                    }
                }),
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.CONSUMPTION_TRESHOLD_TABLE_ENTRY,
                    url: '/json/device/getSensorEventTableEntry',
                    params:
                    {
                        dsid: record.get('dsid'),
                        eventIndex: 0
                    }
                }),
                new dSS.devprops.Loader({
                    loaderId: dSS.devprops.Loaders.CONSUMPTION_TRESHOLD_TABLE_ENTRY,
                    url: '/json/device/getSensorEventTableEntry',
                    params:
                    {
                        dsid: record.get('dsid'),
                        eventIndex: 1
                    }
                })
            ];


        var standardForms =
            [
                {
                    xtype: 'currentconsumptionform',
                    dsid: record.get('dsid')
                },
                {
                    xtype: 'consumptioneventform',
                    dsid: record.get('dsid'),
                    eventIndex: 0,
                    deviceName: record.get('name')
                },
                {
                    xtype: 'consumptioneventform',
                    dsid: record.get('dsid'),
                    eventIndex: 1,
                    deviceName: record.get('name')
                }
            ];

     var retObj =
     {
        panel : new dSS.devprops.DevicePropertiesPanel({
                    dsid: record.get('dsid'),
                    name: record.get('name'),
                    showDefaults: true,
                    standardForms: standardForms,
                    loaders: thresholdloaders,
                    useTabs: false
                })
     };
     return retObj;
};

dSS.devprops.CreatePanelForRenaming = function(record, featureConfig) {
    var forms = [];
    var loaders = [];
    var deviceType;

    if (featureConfig === dSS.config.FeatureConfig.CIRCUIT) {
        loaders.push(
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.METER_NAME,
                url: '/json/circuit/getName',
                params:
                {
                    id: record.get('dsid')
                }
            })
        );
    } else if (featureConfig === dSS.config.FeatureConfig.ZONE) {
        loaders.push(
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.ZONE_NAME,
                url: '/json/zone/getName',
                params:
                {
                    id: record.get('id')
                }
            })
        );
    } else {
        loaders.push(
            new dSS.devprops.Loader({
                loaderId: dSS.devprops.Loaders.DEVICE_NAME,
                url: '/json/device/getName',
                params:
                {
                    dsid: record.get('dsid')
                }
            })
        );
    }

     var retObj =
     {
        panel : new dSS.devprops.DevicePropertiesPanel({
                    name: record.get('name'),
                    showDefaults: false,
                    standardForms:
                    [
                        {
                            xtype: 'dssgenericrenameform',
                            featureConfig: featureConfig,
                            deviceRecord: record
                        }
                    ],
                    loaders: loaders,
                    useTabs: false
                })
     };
    return retObj;
};

dSS.devprops.CreatePanelForEditingGroups = function(record)
{
     var fixedColor = false;
     var deviceStore = dSS.data.Loader.getDeviceStore();
     if (deviceStore.getRange().length > 0) {
         fixedColor = true;
     }

     var retObj =
     {
        panel : new dSS.devprops.DevicePropertiesPanel({
                    name: record.get('name'),
                    showDefaults: false,
                    standardForms:
                    [
                        {
                            xtype: 'dsseditgroupform',
                            groupName: record.get('name'),
                            groupID: record.get('id'),
                            groupColor: record.get('color'),
                            fixedColor: fixedColor
                        }
                    ],
                    useTabs: false
                })
     };
    return retObj;
};

dSS.devprops.CreatePanelForAddingGroups = function(record)
{
    var retObj =
    {
       panel : new dSS.devprops.DevicePropertiesPanel({
                   showDefaults: false,
                   standardForms:
                   [
                       {
                           xtype: 'dssaddgroupform'
                       }
                   ],
                   useTabs: false
               })
    };
    return retObj;
};



dSS.devprops.CreatePropertiesPanel = function(dialogType, record,
                                              featureConfig, scope) {
    switch (dialogType) {
        case dSS.config.dialogTypes.DEVICE_PROPERTIES:
            return dSS.devprops.CreatePanelForDeviceProperties(record,
                                                              featureConfig,
                                                              scope);
        case dSS.config.dialogTypes.SHADE_PROPERTIES:
            return dSS.devprops.CreatePanelForShadeProperties(record);
        case dSS.config.dialogTypes.CONSUMPTION_SETTINGS:
            return dSS.devprops.CreatePanelForConsumptionSettings(record);
        case dSS.config.dialogTypes.RENAME_DIALOG:
            return dSS.devprops.CreatePanelForRenaming(record, featureConfig);
        case dSS.config.dialogTypes.EDIT_GROUP_DIALOG:
            return dSS.devprops.CreatePanelForEditingGroups(record);
        case dSS.config.dialogTypes.CREATE_GROUP_DIALOG:
            return dSS.devprops.CreatePanelForAddingGroups(record);
        default:
            break;
    }
    // if this happened - there was an illegal use of the framework i.e.
    // programming error, user can't configure anything to ever get here
    return undefined;
};
