Ext.namespace("dSS", "dSS.config", "dSS.devprops");
        // configuration values are set up and saved from the advanced form,
        // the standard form gives hints to the advanced form configuration

dSS.devprops.PBConfig = {
    "PUSHBUTTON_AREA1"  : -1,
    "PUSHBUTTON_AREA2"  : -2,
    "PUSHBUTTON_AREA3"  : -3,
    "PUSHBUTTON_AREA4"  : -4,
    "PUSHBUTTON_DEVICE" : -5,
    "PUSHBUTTON_ZONE"   : -6,
    "PUSHBUTTON_AREA"   : -7,
    "PUSHBUTTON_HIGHLEVEL"          : -8,
    "PUSHBUTTON_PANIC"  : -9,
    "PUSHBUTTON_LEAVING_HOME"       : -10,
    "PUSHBUTTON_DOOR_BELL"          : -11,
    "PUSHBUTTON_ALARM"              : -12
};

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dsid
dSS.devprops.GroupPushbuttonForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }
        this.dsid = config.dsid;
        this.group = config.group;
        this.product = config.product;

        if (config.matrix === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("GroupPushbuttonForm: " +
                                                  "missing matrix"));
            return;
        }
        this.matrix = config.matrix;

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.areaSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 100,
            x: 255,
            y: 0,
            mode: 'local',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data:
                [
                    [dSS.devprops.PBConfig.PUSHBUTTON_AREA1, dSS._("Area 1")],
                    [dSS.devprops.PBConfig.PUSHBUTTON_AREA2, dSS._("Area 2")],
                    [dSS.devprops.PBConfig.PUSHBUTTON_AREA3, dSS._("Area 3")],
                    [dSS.devprops.PBConfig.PUSHBUTTON_AREA4, dSS._("Area 4")]
                ]
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        var pbData = [];

        if (this.group !== dSS.config.groups.JOKER) {
            pbData =
            [
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_ZONE,
                    dSS._("zone pushbutton")
                ],
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_AREA,
                    dSS._("area pushbutton")
                ]
            ];
        }

        if (config.matrix.pushbdevice === true) {
            pbData =
            [
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_DEVICE,
                    dSS._("device pushbutton")
                ]
            ].concat(pbData);
        }

        if ((config.matrix.highlevel === true) &&
            (this.group === dSS.config.groups.JOKER)) {
            pbData =
            [
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL,
                    dSS._("server button")
                ]
            ].concat(pbData);
        }

        if (this.group === dSS.config.groups.JOKER) {
            pbData =
            [
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME,
                    dSS._("leaving home")
                ],
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL,
                    dSS._("door bell")
                ],
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_PANIC,
                    dSS._("panic")
                ],
                [
                    dSS.devprops.PBConfig.PUSHBUTTON_ALARM,
                    dSS._("alarm")
                ]
            ].concat(pbData);
        }

        var devStore = dSS.data.Loader.getDeviceStore();
        var groupStore = dSS.data.Loader.getGroupStore();
        var rec = devStore.getById(this.dsid);
        if (rec) {
            var groups = rec.get('groups');
            Ext.each(groups, function(group) {
                if (group < dSS.config.UserGroupStartIndex) {
                    return true;
                }

                var gr = groupStore.getById(group);
                pbData.push([ group,
                              gr.get('name') + " (" + dSS._("Group") + ")" ]);
            });
        }

        this.pbCombo = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 145,
            fieldLabel: dSS._("Pushbutton"),
            mode: 'local',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data: pbData
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    height: 25,
                    width: 355,
                    layout: 'absolute',
                    items:
                    [
                        {
                            xtype: 'panel',
                            border: false,
                            width: 355,
                            x: 0,
                            y: 0,
                            layout: 'form',
                            items: [ this.pbCombo ]
                        },
                        this.areaSelection
                    ]
                }
            ]
        });

        dSS.devprops.GroupPushbuttonForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
            this.areaSelection.on('select', function() {
                    this.fireEvent('formchanged');
                },
                this);

            this.pbCombo.on('select', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.pbCombo.markInvalid(data.json.message);
                    this.fireEvent('formloaded', true);
                    return;
                }

                var modSelection = function(val) {
                    var store = this.pbCombo.getStore();
                    var selrec = store.getById(dSS.devprops.PBConfig.PUSHBUTTON_ZONE);
                    if (selrec !== undefined) {
                        store.remove(selrec);
                    }
                    selrec = store.getById(dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL);
                    if (selrec === undefined) {
                        if (val) {
                            store.on('load', function() {
                                this.pbCombo.setValue(val);
                            }, this, { single: true });
                        }
                        store.loadData(
                            [
                                [
                                    dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL,
                                    dSS._("server button")
                                ]
                            ], true);
                    } else {
                        if (val) {
                            this.pbCombo.setValue(val);
                        }
                    }
                }.createDelegate(this);

                var val = data.json.result[this.dsid][0].button[0].id;
                var activeGroup =
                    data.json.result[this.dsid][0].button[0].activeGroup;
                try {
                    var dec = this.decodeConfig(val);
                    if (activeGroup >= dSS.config.UserGroupStartIndex) {
                        dec[0] = activeGroup;
                        dec[1] = undefined;
                    }
                    this.pbCombo.setValue(dec[0]);
                    // preset hack for some fw that was missing defaults
                    if ((this.matrix.highlevel === true) &&
                        (this.group == dSS.config.groups.JOKER) &&
                        ((dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) &&
                         (dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_DEVICE) &&
                         (dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_PANIC) &&
                         (dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_ALARM) &&
                         (dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL) &&
                         (dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME))) {
                        dec[0] = dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL;
                        this.pbCombo.setValue(dec[0]);
                    }
                    if (dec[0] == dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                        this.areaSelection.setValue(dec[1]);
                    } else if (dec[0] ==
                                dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) {
                        modSelection(dec[0]);
                    }
                    if (dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                        this.areaSelection.disable();
                    }
                    if ((this.group == dSS.config.groups.JOKER) &&
                           ((this.product == dSS.config.deviceTypeNum['TKM']) ||
                            (this.product == dSS.config.deviceTypeNum['KL']) ||
                            (this.product == dSS.config.deviceTypeNum['ZWS']))){
                        modSelection();
                    }

                    // #2288: an invalid button configuration will have no
                    // associated text, it will just be a number, so the
                    // simpliest check is: if the combo selection text evaluates
                    // to a valid number - then the setting is invalid
                    var textValue = this.pbCombo.getRawValue();
                    if (!isNaN(parseInt(textValue, 10))) {
                        this.pbCombo.setValue(
                                this.pbCombo.getStore().getAt(0).get('id'));
                        this.on('loadcomplete', function() {
                            this.fireEvent('formchanged');

                            dSS.MessageBus.fireEvent(
                                'pushbuttonstdformchanged',
                                this.pbCombo.getValue());
                        }, this, { single: true });
                    }
                } catch (ex) {
                    if (this.closed === true) {
                        return;
                    }

                    this.pbCombo.markInvalid(ex);
                }
                this.fireEvent('formloaded', true);
                var setting = this.pbCombo.getValue();
                if (setting === dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                    setting = this.areaSelection.getValue();
                }
                dSS.MessageBus.fireEvent('pushbuttonstdformchanged', setting);
            },
            this);

        if (this.matrix.highlevel === true) {
            dSS.MessageBus.addListener('jokerconfigformchanged',
                                  this.handleServerAppButton, this);
        }

        this.on('beforedestroy', function() {
                this.closed = true;
                if (this.matrix.highlevel === true) {
                    dSS.MessageBus.removeListener('jokerconfigformchanged',
                                           this.handleServerAppButton, this);
                }
            },
            this);

        this.areaSelection.on('select', function(combo, record, index) {
                var v = combo.getValue();

                dSS.MessageBus.fireEvent('pushbuttonstdformchanged', v);
            },
            this);

        this.pbCombo.on('select', function(combo, record, index) {
                var v = combo.getValue();
                if (v == dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                    if (this.areaSelection.getRawValue() == "") {
                        this.areaSelection.setValue(dSS.devprops.PBConfig.PUSHBUTTON_AREA1);
                    }

                    v = this.areaSelection.getValue();
                    this.areaSelection.enable();
                } else {
                    this.areaSelection.setRawValue("");
                    this.areaSelection.disable();
                }
                dSS.MessageBus.fireEvent('pushbuttonstdformchanged', v);
            },
            this);

        this.on('submit', function() {
            // data will be saved by advanced form
            this.fireEvent('formsubmitted');
        },
        this);
    },

    handleServerAppButton: function(color) {
        if (this.matrix.highlevel !== true) {
            return;
        }

        var val = this.pbCombo.getValue();
        var store = this.pbCombo.getStore();

        var rmRecord = function(id) {
            var rec = store.getById(id);
            if (rec !== undefined) {
                store.remove(rec);
            }
        }.createDelegate(this);

        var addRecord = function(id, text) {
            var rec = store.getById(id);
            if (rec === undefined) {
                store.loadData([ [ id, text ] ], true);
            }
        }.createDelegate(this);

        if (color === dSS.config.groups.JOKER) {
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_ZONE);
            addRecord(dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL,
                      dSS._("server button"));
            addRecord(dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME,
                      dSS._("leaving home"));
            addRecord(dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL,
                      dSS._("door bell"));
            addRecord(dSS.devprops.PBConfig.PUSHBUTTON_PANIC,
                      dSS._("panic"));
            addRecord(dSS.devprops.PBConfig.PUSHBUTTON_ALARM,
                      dSS._("alarm"));
            if (val === dSS.devprops.PBConfig.PUSHBUTTON_ZONE) {
                this.pbCombo.setValue(dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL);
                dSS.MessageBus.fireEvent('pushbuttonhighlevelchanged', true);
            } else if (val === dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                if (this.matrix.pushbdevice === true) {
                    this.pbCombo.setValue(dSS.devprops.PBConfig.PUSHBUTTON_DEVICE);
                    dSS.MessageBus.fireEvent('pushbuttonhighlevelchanged', false);
                } else {
                    this.pbCombo.setValue(dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL);
                    dSS.MessageBus.fireEvent('pushbuttonhighlevelchanged', true);
                }
            }
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_AREA);
            this.areaSelection.setRawValue("");
            this.areaSelection.disable();
        } else {
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL);
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_PANIC);
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME);
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_ALARM);
            rmRecord(dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL);

            addRecord(dSS.devprops.PBConfig.PUSHBUTTON_ZONE,
                      dSS._("zone pushbutton"));
            addRecord(dSS.devprops.PBConfig.PUSHBUTTON_AREA,
                      dSS._("area pushbutton"));

            if ((val === dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) ||
                (val === dSS.devprops.PBConfig.PUSHBUTTON_PANIC) ||
                (val === dSS.devprops.PBConfig.PUSHBUTTON_ALARM) ||
                (val === dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL) ||
                (val === dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME)) {
                this.pbCombo.setValue(dSS.devprops.PBConfig.PUSHBUTTON_ZONE);
                dSS.MessageBus.fireEvent('pushbuttonhighlevelchanged', false);
            }
        }

        var setting = this.pbCombo.getValue();
        if (setting === dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
            setting = this.areaSelection.getValue();
        }
        dSS.MessageBus.fireEvent('pushbuttonstdformchanged', setting);
    },

    // decode pushbutton settings value, bank 3 offset 1
    decodeConfig: function(value) {
        var rbval;
        var ext;
        switch (value)
        {
            case dSS.config.deviceParams.PUSHBUTTON_DEVICE:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_DEVICE;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_ALARM:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_ALARM;
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_GROUP1:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP5:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA1;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_PANIC:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_PANIC;
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_GROUP2:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP6:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA2;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_LEAVING_HOME:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME;
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_GROUP3:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP7:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA3;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP4:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP8:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA4;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_DOOR_BELL:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL;
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_APARTMENT:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE0:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE1:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE2:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE3:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE4:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_ZONE;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL;
                break;
            default:
                throw dSS._("Unknown pushbutton configuration!");
        }
        return [ rbval, ext ];
    }
});

Ext.reg('dssdevgrouppushbuttonform', dSS.devprops.GroupPushbuttonForm);

// add advanced form, make sure it can communicate with the standard form

dSS.devprops.GroupPushbuttonAdvancedForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;
        this.matrix = config.matrix;
        this.group = config.group;

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.zonePB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE0,
                dSS._("Scenes 0-4"),
                dSS._("Scenes 2-4"),
                dSS._("Scene 1")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE1,
                dSS._("Scenes 10-14"),
                dSS._("Scenes 12-14"),
                dSS._("Scene 11")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE2,
                dSS._("Scenes 20-24"),
                dSS._("Scenes 22-24"),
                dSS._("Scene 21")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE3,
                dSS._("Scenes 30-34"),
                dSS._("Scenes 32-34"),
                dSS._("Scene 31")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE4,
                dSS._("Scenes 40-44"),
                dSS._("Scenes 42-44"),
                dSS._("Scene 41")
            ]
        ];

        this.areaPB =
        [
            // area 1
            [
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP1,
                    dSS._("Scenes 2-4 (default)"),
                    dSS._("Scenes 2-4 (default)"),
                    dSS._("Area 1")
                ],
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP5,
                    dSS._("Scenes 12-14"),
                    dSS._("Scenes 12-14"),
                    dSS._("Area 1")
                ]
            ],
            // area 2
            [
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP2,
                    dSS._("Scenes 2-4"),
                    dSS._("Scenes 2-4"),
                    dSS._("Area 2")
                ],
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP6,
                    dSS._("Scenes 22-24"),
                    dSS._("Scenes 22-24"),
                    dSS._("Area 2")
                ]
            ],
            // area 3
            [
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP3,
                    dSS._("Scenes 2-4"),
                    dSS._("Scenes 2-4"),
                    dSS._("Area 3")
                ],
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP7,
                    dSS._("Scenes 32-34"),
                    dSS._("Scenes 32-34"),
                    dSS._("Area 3")
                ]
            ],
            // area 4
            [
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP4,
                    dSS._("Scenes 2-4"),
                    dSS._("Scenes 2-4"),
                    dSS._("Area 4")
                ],
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP8,
                    dSS._("Scenes 42-44"),
                    dSS._("Scenes 42-44"),
                    dSS._("Area 4")
                ]
            ]
        ];

        this.devicePB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_DEVICE,
                dSS._("Scenes 2-4"),
                dSS._("Scenes 2-4"),
                dSS._("Device")

            ]
        ];

        this.highlevelPB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL, "", "", ""
            ]
        ];

        this.panicPB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_PANIC, "", "", ""
            ]
        ];

        this.leavePB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_LEAVING_HOME, "", "", ""
            ]
        ];

        this.bellPB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_DOOR_BELL, "", "", ""
            ]
        ];

        this.alarmPB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_ALARM, "", "", ""
            ]
        ];

        this.sceneSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            autoSelect: false,
            editable: false,
            width: 170,
            mode: 'local',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText',
                    'label2Text',
                    'label1Text'
                ]
            }),
            valueField: 'id',
            displayField: 'displayText',
            fieldLabel: dSS._("Used scenes")
        });

        this.label1 = new Ext.form.Label({
             xtype: 'label',
             fieldLabel: dSS._("1x Click")
         });

        this.label2 = new Ext.form.Label({
            xtype: 'label',
            fieldLabel: dSS._("2x, 3x, 4x Click")
        });

        this.localPrio = new Ext.form.Checkbox({
            boxLabel: dSS._("Local priority for 1x click")
        });

        this.callsPresent = new Ext.form.Checkbox({
            boxLabel: dSS._("Calls Present on click")
        });

        this.lpTip = new Ext.ToolTip({
            html: dSS._("The local priority settings were changed."),
            title: dSS._("Notice"),
            closable: true,
            draggable: true,
            dismissDelay: 4000,
            anchor: 'left'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            border: false,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    ref: 'main',
                    height: 140,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        {
                            xtype: 'label',
                            text: dSS._("Pushbutton") + ':'
                        },
                        this.sceneSelection,
                        this.label1,
                        this.label2,
                        this.localPrio,
                        this.callsPresent
                    ]
                }
            ]
        });

        dSS.devprops.GroupPushbuttonAdvancedForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.callsPresent.on('check', function() {
                    this.fireEvent('formchanged');
                },
                this);

                this.localPrio.on('check', function() {
                    this.fireEvent('formchanged');
                },
                this);

                this.sceneSelection.on('select', function() {
                        this.fireEvent('formchanged');
                },
                this);
            },
            this);

        dSS.MessageBus.addListener('pushbuttonstdformchanged',
                          this.updateForm, this);

        dSS.MessageBus.addListener('jokerconfigformchanged',
                          this.updateLocalPriority, this);

        this.sceneSelection.on('select', function(combo, record, index) {
                this.label1.setText(record.get('label1Text'));
                this.label2.setText(record.get('label2Text'));
            },this);

        this.loadedValue = undefined;
        this.loadedPrioLocalScene = undefined;
        this.loadedCallsPresent = undefined;
        this.currentValue = undefined;
        this.loadedActiveGroup = undefined;
        this.activeGroup = undefined;

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }
                if (data.loader == dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    if (data.json.ok !== true) {
                        this.sceneSelection.markInvalid(data.json.message);
                        this.label1.setText("");
                        this.label2.setText("");
                        this.localPrio.setDisabled(true);
                        this.callsPresent.setDisabled(true);
                        this.fireEvent('formloaded', true);
                        return;
                    }

                    try {
                        var val = data.json.result[this.dsid][0].button[0].id;
                        var prio = data.json.result[this.dsid][0].button[0].setsLocalPriority;
                        var callsPres = data.json.result[this.dsid][0].button[0].callsPresent;
                        this.loadedActiveGroup =
                           data.json.result[this.dsid][0].button[0].activeGroup;
                        this.activeGroup = this.loadedActiveGroup;

                        var load = this.decodeConfig(val);
                        this.loadedValue = load[1];
                        // preset hack for fw without defaults
                        if ((this.matrix.highlevel === true) &&
                            (this.group == dSS.config.groups.JOKER) &&
                            ( ((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL) &&
                             ((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_PANIC) &&
                             ((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_LEAVING_HOME) &&
                             ((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_ALARM) &&
                             ((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_DOOR_BELL) &&
                             ((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_DEVICE))) {
                            val = dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL;
                            load = this.decodeConfig(val);
                        }
                        this.sceneSelection.getStore().loadData(load[0]);
                        this.sceneSelection.setValue(load[1]);
                        this.label1.setText(load[2]);
                        this.label2.setText(
                            this.sceneSelection.getStore().getById(load[1]).get(
                                                                    'displayText'));
                        this.currentValue = load[4];
                        this.loadedPrioAreas = prio;
                        this.localPrio.setValue(prio);
                        this.localPrio.setDisabled(!load[3]);
                        this.loadedCallsPresent = callsPres;
                        this.callsPresent.setValue(callsPres);
                        this.callsPresent.setDisabled(false);

                        if (load[4] ==
                                dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) {
                            this.disableElements();
                        }
                    } catch (ex) {
                        if (this.closed === true) {
                            return;
                        }

                        this.sceneSelection.markInvalid(ex);
                        this.label1.setText("");
                        this.label2.setText("");
                    }
                } else if (data.loader == dSS.devprops.Loaders.SCENE_CONFIGURATION) {
                    if (data.json.ok !== true) {
                        this.sceneSelection.markInvalid(data.json.message);
                        this.label1.setText("");
                        this.label2.setText("");
                        this.localPrio.setDisabled(true);
                        this.callsPresent.setDisabled(true);
                        this.fireEvent('formloaded', true);
                        return;
                    }

                    try {
                        this.loadedPrioLocalScene = data.json.result.specialMode;
                        if (this.loadedValue === dSS.config.deviceParams.PUSHBUTTON_DEVICE) {
                            this.localPrio.setValue((this.loadedPrioLocalScene) ? 1 : 0);
                            this.localPrio.setDisabled(false);
                        }
                    } catch (ex) {
                        if (this.closed === true) {
                            return;
                        }
                    }
                }

                if ((this.loadedValue !== undefined) &&
                    (this.loadedPrioLocalScene !== undefined)) {
                    this.fireEvent('formloaded', true);
                }
            },
            this);

        this.on('beforedestroy', function() {
                dSS.MessageBus.removeListener('pushbuttonstdformchanged',
                                              this.updateForm, this);
                dSS.MessageBus.removeListener('jokerconfigformchanged',
                                              this.updateLocalPriority, this);
                this.closed = true;
            },
            this);

        this.on('submit', function() {
            var value = this.sceneSelection.getValue();
            var prio = this.localPrio.getValue();
            var callsPresent = this.callsPresent.getValue();

            if (!((value !== this.loadedValue) ||
                  (this.activeGroup !== this.loadedActiveGroup)) &&
                !(callsPresent != this.loadedCallsPresent) &&
                !((value === dSS.config.deviceParams.PUSHBUTTON_DEVICE) && (prio != this.loadedPrioLocalScene)) &&
                !((value !== dSS.config.deviceParams.PUSHBUTTON_DEVICE) && (prio != this.loadedPrioAreas))) {
                this.fireEvent('formsubmitted');
                return;
            }

            var id = this.dsid;
            var deferDelay = 1000;
            var eventSent = false;

            if ((value !== this.loadedValue) &&
                ((this.activeGroup === dSS.config.ResetActiveButtonGroup) ||
                 (this.activeGroup < dSS.config.UserGroupStartIndex))) {
                eventSent = true;
                // we know this form is never used alone, it's always in the
                // same dialog as the output mode form; in order to not spam
                // the bus with two simultanious requests we will delay saving
                // of pushbutton settings by one second
                dSS.util.genericRequest.defer(deferDelay,
                    this,
                    [
                        {
                            url: '/json/device/setButtonID',
                            params:
                            {
                                dsid : this.dsid,
                                buttonID : value
                            },
                            failureMsg:
                                    dSS._("Could not save pushbutton settings"),
                            scope: this,
                            evt: 'formsubmitted'
                        },
                        function(success) {
                            if (success === true) {
                                var s = dSS.data.Loader.getDeviceStore();
                                var rec = s.getById(id);
                                if (rec !== undefined) {
                                    rec.set('buttonID', value);
                                    rec.commit();
                                    dSS.MessageBus.fireEvent('scenesmarkdirty');
                                }
                            }
                        }
                    ]
                );
                deferDelay += 500;
            }

            if ((this.activeGroup >= dSS.config.UserGroupStartIndex) &&
                (this.activeGroup !== this.loadedActiveGroup)) {
                eventSent = true;

                if ((this.activeGroup < dSS.config.UserGroupStartIndex) &&
                    (this.activeGroup < this.loadedActiveGroup)) {
                    this.activeGroup = dSS.config.ResetActiveButtonGroup;
                }
                var ag = this.activeGroup;
                dSS.util.genericRequest.defer(deferDelay,
                    this,
                    [
                        {
                            url: '/json/device/setButtonActiveGroup',
                            params:
                            {
                                dsid : this.dsid,
                                groupID : this.activeGroup
                            },
                            failureMsg:
                                    dSS._("Could not save user group settings"),
                            scope: this,
                            evt: 'formsubmitted'
                        },
                        function(success) {
                            if (success === true) {
                                var s = dSS.data.Loader.getDeviceStore();
                                var rec = s.getById(id);
                                if (rec !== undefined) {
                                    rec.set('buttonActiveGroup', ag);
                                    rec.commit();
                                    dSS.MessageBus.fireEvent('scenesmarkdirty');
                                }
                            }
                        }
                    ]
                );
                deferDelay += 500;
            }

            if (value === dSS.config.deviceParams.PUSHBUTTON_DEVICE) {
                if (prio != this.loadedPrioLocalScene) {
                    eventSent = true;
                    dSS.util.genericRequest.defer(deferDelay,
                        this,
                        [
                            {
                                url: '/json/device/setSceneMode',
                                params:
                                {
                                    sceneID : dSS.config.scenes.LOCAL_ON,
                                    specialMode: prio ? 1 : 0,
                                    dsid: this.dsid
                                },
                                failureMsg:
                                        dSS._("Could not save pushbutton settings"),
                                scope: this,
                                evt: 'formsubmitted'
                            }
                        ]
                    );
                }
            } else {
                if (prio != this.loadedPrioAreas) {
                    eventSent = true;
                    dSS.util.genericRequest.defer(deferDelay,
                        this,
                        [
                            {
                                url: '/json/property/setBoolean',
                                params:
                                {
                                    path: '/apartment/zones/zone0/devices/' +
                                          this.dsid +
                                          '/button/setsLocalPriority',
                                    value: prio
                                },
                                failureMsg:
                                        dSS._("Could not save pushbutton settings"),
                                scope: this,
                                evt: 'formsubmitted'
                            }
                        ]
                    );
                }
            }
            if (callsPresent != this.loadedCallsPresent) {
                eventSent = true;
                dSS.util.genericRequest.defer(deferDelay,
                    this,
                    [
                        {
                            url: '/json/property/setBoolean',
                            params:
                            {
                                path: '/apartment/zones/zone0/devices/' +
                                      this.dsid +
                                      '/button/callsPresent',
                                value: callsPresent
                            },
                            failureMsg:
                                    dSS._("Could not save pushbutton settings"),
                            scope: this,
                            evt: 'formsubmitted'
                        }
                    ]
                );
            }

            if (eventSent !== true) {
                this.fireEvent('formsubmitted');
            }
        },
        this);
    },
    // decode pushbutton settings value, bank 3 offset 1
    decodeConfig: function(value) {
        var ret;
        var l1;
        var showLocalPriority = false;
        var rbval;
        switch (value)
        {
            case dSS.config.deviceParams.PUSHBUTTON_DEVICE:
                ret = this.devicePB;
                l1 = dSS._("Device");
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_DEVICE;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_ALARM:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_ALARM;
                    ret = this.alarmPB;
                    l1 = "";
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_GROUP1:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP5:
                ret = this.areaPB[0];
                l1 = dSS._("Area 1");
                showLocalPriority = true;
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA1;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP2:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_PANIC;
                    ret = this.panicPB;
                    l1 = "";
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_GROUP6:
                ret = this.areaPB[1];
                l1 = dSS._("Area 2");
                showLocalPriority = true;
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA2;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_LEAVING_HOME:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME;
                    ret = this.leavePB;
                    l1 = "";
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_GROUP3:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP7:
                ret = this.areaPB[2];
                l1 = dSS._("Area 3");
                showLocalPriority = true;
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA3;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP4:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP8:
                ret = this.areaPB[3];
                l1 = dSS._("Area 4");
                showLocalPriority = true;
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA4;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_DOOR_BELL:
                if (this.group === dSS.config.groups.JOKER) {
                    rbval = dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL;
                    ret = this.bellPB;
                    l1 = "";
                    break;
                }
            case dSS.config.deviceParams.PUSHBUTTON_APARTMENT:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE0:
                l1 = this.zonePB[0][3];
            case dSS.config.deviceParams.PUSHBUTTON_ZONE1:
                if (l1 === undefined) {
                    l1 = this.zonePB[1][3];
                }
            case dSS.config.deviceParams.PUSHBUTTON_ZONE2:
                if (l1 === undefined) {
                    l1 = this.zonePB[2][3];
                }
            case dSS.config.deviceParams.PUSHBUTTON_ZONE3:
                if (l1 === undefined) {
                    l1 = this.zonePB[3][3];
                }
            case dSS.config.deviceParams.PUSHBUTTON_ZONE4:
                if (l1 === undefined) {
                    l1 = this.zonePB[4][3];
                }
                ret = this.zonePB;
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_ZONE;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL;
                ret = this.highlevelPB;
                l1 = "";
                break;
            default:
                throw dSS._("Unknown pushbutton configuration!");

        }
        return [ ret, value, l1, showLocalPriority, rbval ];
    },
    updateLocalPriority: function(color) {
        if ((this.group != color) &&
            (color != dSS.config.groups.JOKER) &&
            (this.group != dSS.config.groups.JOKER) &&
            ((this.group == dSS.config.groups.SHADOW) ||
             (color == dSS.config.groups.SHADOW))) {
            this.group = color;
            if (this.updateLocalPrio()) {
                if ((this.ownerCt !== undefined) &&
                    (this.ownerCt.xtype === 'panel') &&
                    (this.ownerCt.collapsed === true)) {
                        this.lpTip.initTarget(this.ownerCt.getId());
                        this.lpTip.enable();
                        this.lpTip.show();
                }
            }
        } else {
            this.group = color;
            // Hide tooltip again
            if ((this.ownerCt !== undefined) &&
                (this.ownerCt.xtype === 'panel')) {
                    this.lpTip.disable();
                    this.lpTip.hide();
            }
        }
    },
    enableElements: function() {
        this.sceneSelection.enable();
        this.label1.enable();
        this.label2.enable();
    },
    disableElements: function() {
        this.sceneSelection.disable();
        this.label1.disable();
        this.label2.disable();
    },
    updateForm: function(value) {
        if ((value < 0) && (this.loadedActiveGroup >=
                dSS.config.UserGroupStartIndex)) {
            this.activeGroup = dSS.config.ResetActiveButtonGroup;
        }
        this.currentValue = value;
        if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA1) {
                this.sceneSelection.getStore().loadData(this.areaPB[0]);
                this.sceneSelection.setValue(this.areaPB[0][0][0]);
                this.label1.setText(dSS._("Area 1"));
                this.label2.setText(this.areaPB[0][0][1]);
                this.enableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA2) {
                this.sceneSelection.getStore().loadData(this.areaPB[1]);
                this.sceneSelection.setValue(this.areaPB[1][0][0]);
                this.label1.setText(dSS._("Area 2"));
                this.label2.setText(this.areaPB[1][0][1]);
                this.enableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA3) {
                this.sceneSelection.getStore().loadData(this.areaPB[2]);
                this.sceneSelection.setValue(this.areaPB[2][0][0]);
                this.label1.setText(dSS._("Area 3"));
                this.label2.setText(this.areaPB[2][0][1]);
                this.enableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA4) {
                this.sceneSelection.getStore().loadData(this.areaPB[3]);
                this.sceneSelection.setValue(this.areaPB[3][0][0]);
                this.label1.setText(dSS._("Area 4"));
                this.label2.setText(this.areaPB[3][0][1]);
                this.enableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_DEVICE) {
                this.sceneSelection.getStore().loadData(this.devicePB);
                this.sceneSelection.setValue(this.devicePB[0][0]);
                this.label1.setText(dSS._("Device"));
                this.label2.setText(this.devicePB[0][1]);
                this.enableElements();
        } else if ((value == dSS.devprops.PBConfig.PUSHBUTTON_ZONE) ||
                   (value == dSS.devprops.PBConfig.PUSHBUTTON_APARTMENT) ||
                   (value >= dSS.config.UserGroupStartIndex)) {
                this.sceneSelection.getStore().loadData(this.zonePB);
                this.sceneSelection.setValue(this.zonePB[0][0]);
                this.label1.setText(this.zonePB[0][3]);
                this.label2.setText(this.zonePB[0][2]);
                this.enableElements();
                if (value >= dSS.config.UserGroupStartIndex) {
                    this.activeGroup = value;
                }
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) {
                this.sceneSelection.getStore().loadData(this.highlevelPB);
                this.sceneSelection.setValue(this.highlevelPB[0][0]);
                this.label1.setText("");
                this.label2.setText("");
                this.disableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_PANIC) {
                this.sceneSelection.getStore().loadData(this.panicPB);
                this.sceneSelection.setValue(this.panicPB[0][0]);
                this.label1.setText("");
                this.label2.setText("");
                this.disableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_LEAVING_HOME) {
                this.sceneSelection.getStore().loadData(this.leavePB);
                this.sceneSelection.setValue(this.leavePB[0][0]);
                this.label1.setText("");
                this.label2.setText("");
                this.disableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_DOOR_BELL) {
                this.sceneSelection.getStore().loadData(this.bellPB);
                this.sceneSelection.setValue(this.bellPB[0][0]);
                this.label1.setText("");
                this.label2.setText("");
                this.disableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_ALARM) {
                this.sceneSelection.getStore().loadData(this.alarmPB);
                this.sceneSelection.setValue(this.alarmPB[0][0]);
                this.label1.setText("");
                this.label2.setText("");
                this.disableElements();
        } else {
                this.sceneSelection.getStore().loadData([]);
                this.sceneSelection.setValue();
                this.sceneSelection.markInvalid(
                        dSS._("Unknown pushbutton configuration!"));
                this.label1.setText("");
                this.label2.setText("");
        }
        this.updateLocalPrio();
    },
    updateLocalPrio: function() {
        var value = this.currentValue;
        if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA1) {
            this.localPrio.setDisabled(false);
            if (this.group == dSS.config.groups.SHADOW) {
                this.localPrio.setValue(0);
            } else {
                this.localPrio.setValue(1);
            }
            return true;
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA2) {
            this.localPrio.setDisabled(false);
            if (this.group == dSS.config.groups.SHADOW) {
                this.localPrio.setValue(0);
            } else {
                this.localPrio.setValue(1);
            }
            return true;
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA3) {
            this.localPrio.setDisabled(false);
            if (this.group == dSS.config.groups.SHADOW) {
                this.localPrio.setValue(0);
            } else {
                this.localPrio.setValue(1);
            }
            return true;
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA4) {
            this.localPrio.setDisabled(false);
            if (this.group == dSS.config.groups.SHADOW) {
                this.localPrio.setValue(0);
            } else {
                this.localPrio.setValue(1);
            }
            return true;
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_DEVICE) {
            this.localPrio.setDisabled(false);
            this.localPrio.setValue((this.loadedPrioLocalScene) ? 1 : 0);
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_ZONE) {
            this.localPrio.setDisabled(true);
            this.localPrio.setValue(0);
        } else {
            this.localPrio.setDisabled(true);
        }
        return false;
    }
});

Ext.reg('dssdevgrouppushbuttonadvancedform', dSS.devprops.GroupPushbuttonAdvancedForm);

